﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Mail;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SMTP_Client {

  public partial class FormMaster : EZ_Builder.UCForms.FormPluginMaster {

    public FormMaster() {

      InitializeComponent();
    }

    public override void SetConfiguration(EZ_Builder.Config.Sub.PluginV1 cf) {

      cf.STORAGE.AddIfNotExist(ConfigDictionary._FROM_NAME, string.Empty);
      cf.STORAGE.AddIfNotExist(ConfigDictionary._FROM_EMAIL, string.Empty);
      cf.STORAGE.AddIfNotExist(ConfigDictionary._PASSWORD, string.Empty);
      cf.STORAGE.AddIfNotExist(ConfigDictionary._SMTP_ADDRESS, string.Empty);
      cf.STORAGE.AddIfNotExist(ConfigDictionary._SMTP_PORT, 25);
      cf.STORAGE.AddIfNotExist(ConfigDictionary._USE_AUTHENTICATION, false);
      cf.STORAGE.AddIfNotExist(ConfigDictionary._USERNAME, string.Empty);
      cf.STORAGE.AddIfNotExist(ConfigDictionary._USE_SSL, false);

      base.SetConfiguration(cf);
    }

    private void sendEmail(string email, string name, string subject, string message, string cameraControlName) {

      EZ_Builder.Invokers.SetAppendText(textBox1, true, "To: {0}", email);

      EZ_Builder.UCForms.FormCameraDevice [] forms = EZ_Builder.EZBManager.FormMain.GetControlByType(typeof(EZ_Builder.UCForms.FormCameraDevice)).Cast<EZ_Builder.UCForms.FormCameraDevice>().ToArray();

      foreach (var form in forms) {

        if (form.Text.Equals(cameraControlName, StringComparison.InvariantCultureIgnoreCase)) {

          using (MemoryStream ms = new MemoryStream()) {

            form.Camera.GetCurrentBitmap.Save(ms, System.Drawing.Imaging.ImageFormat.Jpeg);

            ms.Position = 0;
            
            using (LinkedResource imageResource = new LinkedResource(ms, System.Net.Mime.MediaTypeNames.Image.Jpeg)) {

              imageResource.ContentId = Guid.NewGuid().ToString();

              string htmlBody = message + @"<br /><br /><img src='cid:" + imageResource.ContentId + @"'/>";

              using (AlternateView alternateView = AlternateView.CreateAlternateViewFromString(htmlBody, null, System.Net.Mime.MediaTypeNames.Text.Html)) {

                alternateView.LinkedResources.Add(imageResource);
                
                using (SmtpClient client = new SmtpClient()) {

                  client.EnableSsl = Convert.ToBoolean(_cf.STORAGE[ConfigDictionary._USE_SSL]);

                  if (Convert.ToBoolean(_cf.STORAGE[ConfigDictionary._USE_AUTHENTICATION])) {

                    client.DeliveryMethod = SmtpDeliveryMethod.Network;

                    client.UseDefaultCredentials = false;

                    client.Credentials = new NetworkCredential(
                      _cf.STORAGE[ConfigDictionary._USERNAME].ToString(),
                      _cf.STORAGE[ConfigDictionary._PASSWORD].ToString());
                  }

                  client.Host = _cf.STORAGE[ConfigDictionary._SMTP_ADDRESS].ToString();

                  client.Port = Convert.ToInt32(_cf.STORAGE[ConfigDictionary._SMTP_PORT]);

                  using (MailMessage mm = new MailMessage()) {

                    mm.To.Add(new MailAddress(email, name));

                    mm.From = new MailAddress(
                        _cf.STORAGE[ConfigDictionary._FROM_EMAIL].ToString(),
                        _cf.STORAGE[ConfigDictionary._FROM_NAME].ToString());

                    mm.Subject = subject;

                    mm.IsBodyHtml = true;

                    mm.AlternateViews.Add(alternateView);

                    client.Send(mm);

                    return;
                  }
                }
              }
            }
          }
        }
      }

      throw new Exception(string.Format("Unable to find a camera control named '{0}'", cameraControlName));
    }

    private void sendEmail(string email, string name, string subject, string message) {

      EZ_Builder.Invokers.SetAppendText(textBox1, true, "To: {0}", email);

      using (SmtpClient client = new SmtpClient()) {

        client.EnableSsl = Convert.ToBoolean(_cf.STORAGE[ConfigDictionary._USE_SSL]);

        if (Convert.ToBoolean(_cf.STORAGE[ConfigDictionary._USE_AUTHENTICATION])) {

          client.DeliveryMethod = SmtpDeliveryMethod.Network;

          client.UseDefaultCredentials = false;

          client.Credentials = new NetworkCredential(
            _cf.STORAGE[ConfigDictionary._USERNAME].ToString(),
            _cf.STORAGE[ConfigDictionary._PASSWORD].ToString());
        }

        client.Host = _cf.STORAGE[ConfigDictionary._SMTP_ADDRESS].ToString();

        client.Port = Convert.ToInt32(_cf.STORAGE[ConfigDictionary._SMTP_PORT]);

        using (MailMessage mm = new MailMessage()) {

          mm.To.Add(new MailAddress(email, name));

          mm.From = new MailAddress(
              _cf.STORAGE[ConfigDictionary._FROM_EMAIL].ToString(),
              _cf.STORAGE[ConfigDictionary._FROM_NAME].ToString());

          mm.Subject = subject;

          mm.IsBodyHtml = false;

          mm.Body = message;

          client.Send(mm);
        }
      }
    }

    public override void SendCommand(string windowCommand, params string[] values) {

      if (windowCommand.Equals(ControlCommandDictionary.SEND, StringComparison.InvariantCultureIgnoreCase)) {

        if (values.Length != 4)
          throw new Exception(string.Format("Expecting 4 parameters, you passed {0}", values.Length));

        string email   = values[0];
        string name    = values[1];
        string subject = values[2];
        string message = values[3];

        sendEmail(email, name, subject, message);
      } else if (windowCommand.Equals(ControlCommandDictionary.SEND_IMAGE, StringComparison.InvariantCultureIgnoreCase)) {

        if (values.Length != 5)
          throw new Exception(string.Format("Expecting 5 parameters, you passed {0}", values.Length));

        string cameraName = values[0];
        string email      = values[1];
        string name       = values[2];
        string subject    = values[3];
        string message    = values[4];

        sendEmail(email, name, subject, message, cameraName);
      } else {

        base.SendCommand(windowCommand, values);
      }
    }

    public override object[] GetSupportedControlCommands() {

      List<string> cmds = new List<string>();

      cmds.Add(string.Format("{0}, [recipient email], [recipient name], [subject], [message]", ControlCommandDictionary.SEND));

      cmds.Add(string.Format("{0}, [camera control name], [recipient email], [recipient name], [subject], [message]", ControlCommandDictionary.SEND_IMAGE));

      foreach (var form in EZ_Builder.EZBManager.FormMain.GetControlByType(typeof(EZ_Builder.UCForms.FormCameraDevice)))
        cmds.Add(string.Format("{0}, \"{1}\", [recipient email], [recipient name], [subject], [message]", ControlCommandDictionary.SEND_IMAGE, form.Text));

      return cmds.ToArray();
    }

    private void ucConfigurationButton1_Click(object sender, EventArgs e) {

      using (FormConfig form = new FormConfig()) {

        form.SetConfiguration(_cf);

        if (form.ShowDialog() == System.Windows.Forms.DialogResult.OK)
          _cf = form.GetConfiguration();
      }
    }
  }
}
